@@ -1,6 +1,9 @@
 import inspect
 import re
+import sys
 import typing as t
+from pathlib import Path
+from types import ModuleType
 
 try:
     from typing import Literal
@@ -19,23 +22,34 @@ from scanpydoc.elegant_typehints.formatting import (
 from scanpydoc.elegant_typehints.return_tuple import process_docstring
 
 
-TestCls = type("Class", (), {})
-TestCls.__module__ = "_testmod"
-TestExc = type("Excep", (RuntimeError,), {})
-TestExc.__module__ = "_testmod"
+@pytest.fixture
+def _testmod(make_module):
+    return make_module(
+        "_testmod",
+        """\
+        class Class: pass
+        class SubCl(Class): pass
+        class Excep(RuntimeError): pass
+        class Excep2(Excep): pass
+        """,
+    )
 
 
 @pytest.fixture
 def app(make_app_setup) -> Sphinx:
     return make_app_setup(
+        master_doc="index",
         extensions=[
+            "sphinx.ext.autodoc",
             "sphinx.ext.napoleon",
             "sphinx_autodoc_typehints",
             "scanpydoc.elegant_typehints",
         ],
         qualname_overrides={
             "_testmod.Class": "test.Class",
+            "_testmod.SubCl": "test.SubCl",
             "_testmod.Excep": "test.Excep",
+            "_testmod.Excep2": "test.Excep2",
         },
     )
 
@@ -142,21 +156,21 @@ def test_literal(app):
     )
 
 
-def test_qualname_overrides_class(app):
-    assert TestCls.__module__ == "_testmod"
-    assert _format_terse(TestCls) == ":py:class:`~test.Class`"
+def test_qualname_overrides_class(app, _testmod):
+    assert _testmod.Class.__module__ == "_testmod"
+    assert _format_terse(_testmod.Class) == ":py:class:`~test.Class`"
 
 
-def test_qualname_overrides_exception(app):
-    assert TestExc.__module__ == "_testmod"
-    assert _format_terse(TestExc) == ":py:exc:`~test.Excep`"
+def test_qualname_overrides_exception(app, _testmod):
+    assert _testmod.Excep.__module__ == "_testmod"
+    assert _format_terse(_testmod.Excep) == ":py:exc:`~test.Excep`"
 
 
-def test_qualname_overrides_recursive(app):
-    assert _format_terse(t.Union[TestCls, str]) == (
+def test_qualname_overrides_recursive(app, _testmod):
+    assert _format_terse(t.Union[_testmod.Class, str]) == (
         r":py:class:`~test.Class`, :py:class:`str`"
     )
-    assert _format_full(t.Union[TestCls, str]) == (
+    assert _format_full(t.Union[_testmod.Class, str]) == (
         r":py:data:`~typing.Union`\["
         r":py:class:`~test.Class`, "
         r":py:class:`str`"
@@ -164,11 +178,11 @@ def test_qualname_overrides_recursive(app):
     )
 
 
-def test_fully_qualified(app):
-    assert _format_terse(t.Union[TestCls, str], True) == (
+def test_fully_qualified(app, _testmod):
+    assert _format_terse(t.Union[_testmod.Class, str], True) == (
         r":py:class:`test.Class`, :py:class:`str`"
     )
-    assert _format_full(t.Union[TestCls, str], True) == (
+    assert _format_full(t.Union[_testmod.Class, str], True) == (
         r":py:data:`typing.Union`\[" r":py:class:`test.Class`, " r":py:class:`str`" r"]"
     )
 
@@ -222,6 +236,67 @@ def test_typing_class_nested(app):
     )
 
 
+@pytest.mark.parametrize(
+    "direc,base,sub",
+    [
+        ("autoclass", "Class", "SubCl"),
+        ("autoexception", "Excep", "Excep2"),
+    ],
+)
+def test_autodoc(app, _testmod, direc, base, sub):
+    Path(app.srcdir, "index.rst").write_text(
+        f"""\
+.. {direc}:: _testmod.{sub}
+   :show-inheritance:
+"""
+    )
+    app.build()
+    out = Path(app.outdir, "index.html").read_text()
+    assert not app._warning.getvalue(), app._warning.getvalue()
+    assert re.search(rf"<code[^>]*>test\.</code><code[^>]*>{sub}</code>", out), out
+    assert f'<a class="headerlink" href="#test.{sub}"' in out, out
+    assert re.search(rf"Bases: <code[^>]*><span[^>]*>test\.{base}", out), out
+
+
+@pytest.mark.skipif(sys.version_info < (3, 7), reason="bpo-34776 only fixed on 3.7+")
+def test_fwd_ref(app, make_module):
+    make_module(
+        "fwd_mod",
+        """\
+        from dataclasses import dataclass
+
+        @dataclass
+        class A:
+           b: 'B'
+
+        @dataclass
+        class B:
+            a: A
+        """,
+    )
+    Path(app.srcdir, "index.rst").write_text(
+        f"""\
+.. autosummary::
+
+   fwd_mod.A
+   fwd_mod.B
+"""
+    )
+    app.setup_extension("sphinx.ext.autosummary")
+
+    app.build()
+
+    out = Path(app.outdir, "index.html").read_text()
+    warnings = [
+        w
+        for w in app._warning.getvalue().splitlines()
+        if "Cannot treat a function defined as a local function" not in w
+    ]
+    assert not warnings, warnings
+    # TODO: actually reproduce #14
+    assert "fwd_mod.A" in out, out
+
+
 @pytest.mark.parametrize(
     "docstring",
     [
